import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 * A Canvas class for doing basic graphical drawing.
 * This particular canvas focuses on drawing shapes that have been "added" to it.
 * Acts as a Singleton for ease of use.
 * 
 * @author Joel Ross
 * @author based on class by Michael Kolling and Bruce Quig
 *
 * @version Sp 2015
 */

public class ShapeCanvas
{
    private static final int WIDTH = 400;
    private static final int HEIGHT = 400;

    private static JFrame frame; //the window on which the drawing is displayed
    private static ArrayList<DrawableShape> shapes; //the collection of shapes that are being drawn
    
    private static Graphics2D g2d; //the graphics context we are drawing on
    private static BufferedImage canvasImage; //an "image" that holds the current drawing
    private static JPanel panel; //the component that is drawn inside the frame

    /**
     * Adds the given shape to the canvas.
     * This instantiates the singleton canvas if not already created
     */
    public static void addShape(DrawableShape shape)
    {
        if(frame == null)
            createFrame();
        shapes.add(shape);
    }

    /**
     * Removes the given shape from the canvas
     */
    public static void removeShape(DrawableShape shape)
    {
        shapes.remove(shape);
    }
    
    /**
     * Moves a shape to the back of the drawing
     */
    public static void sendToBack(DrawableShape shape)
    {
        shapes.remove(shape); //pull out of list
        shapes.add(0, shape); //stick in the front (first drawn)
    }
    
    /**
     * Moves a shape to the front of the drawing
     */
    public static void bringToFront(DrawableShape shape)
    {
        shapes.remove(shape); //pull out of list
        shapes.add(shape); //stick in the back (last drawn)
    }
    
    /**
     * Instantiates and initializes the canvas
     */
    private static void createFrame()
    {
        frame = new JFrame(); //new JFrame
        frame.setTitle("Shapes");
        //frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE); //exit program when closed

        //initialize the panel and drawing images
        panel = new JPanel() {
            public void paintComponent(Graphics g) { g.drawImage(canvasImage,0,0,null); }
        };
        panel.setPreferredSize(new Dimension(WIDTH, HEIGHT));
        canvasImage = new BufferedImage(WIDTH, HEIGHT, BufferedImage.TYPE_INT_ARGB);
        g2d = canvasImage.createGraphics();
        g2d.setStroke(new BasicStroke(2)); //thicker lines
        frame.getContentPane().add(panel); //add the panel to the frame

        shapes = new ArrayList<DrawableShape>(); //initialize the list

        frame.pack(); //pack everything
        frame.setVisible(true); //show on the screen

        //allows secret deleting (in case a shape is removed from the workbench)
        panel.addMouseListener(new MouseAdapter(){
        	public void mousePressed(MouseEvent e) {
        		if(!e.isShiftDown() || !e.isControlDown())
        			return;
        		DrawableShape toDelete = null;
        		for(DrawableShape shape : shapes)
        			if(shape.contains(e.getX(), e.getY())) toDelete = shape;
        		shapes.remove(toDelete);
        		refresh();
        	};
        });
    }

    /**
     * A method that refreshes what is displayed on the canvas.
     * This method redraws the shapes on the canvas.
     */
    public static void refresh()
    {
        g2d.setColor(Color.WHITE);
        g2d.fillRect(0,0,canvasImage.getWidth(), canvasImage.getHeight());
        for(DrawableShape shape : shapes)
        {
            shape.draw(g2d);
        }
        panel.repaint();
    }

    /**
     * Shows the canvas frame, in case it was closed.
     */
    public static void makeVisible()
    {
        frame.setVisible(true);
    }
}

